<?php

namespace Service;

use Util\OpensslCrypt;
use Util\Utilities;
use Util\Instance\GeneralInstance;
use Util\Instance\RepositoryInstance;
use Util\Instance\ServiceInstance;
use Exception\Login as LoginException;
use Zend\Mime\Part as ZendMimePart;
use Zend\Mime\Message as ZendMimeMessage;
use Teknisa\Model\Teknisa\Entities\OmConfigMedidaVariavel;

class License
{
    const LIBRARIES_CLIENT_ID                 = 'http://libraries.teknisa.com/';
    const VARIABLE_MEASURE_COLLECTION         = 'VariableMeasure';
    const GENERAL_COLLECTION                  = 'GeneralCollection';
    const VARIABLE_MEASURE_ERROR_STATUS       = 'E';
    const VARIABLE_MEASURE_SUCCESS_STATUS     = 'S';
    const VARIABLE_MEASURE_BASE_SUBJECT_EMAIL = 'Erro na coleta de Medidas Variáveis';

    /**
     * @param $organizationId
     * @param $productId
     * @param $elementIds
     * @param $serverId
     * @param null $user
     * @param null $password
     * @param bool $hasUserData
     * @param bool $returnOrgAndBranches
     * @return array
     * @throws \Zeedhi\Framework\DB\Mongo\Exception
     * @throws \Zeedhi\Framework\Security\OAuth\Exception
     */
    public function getProductLicense($organizationId, $productId, $elementIds, $serverId, $user = null, $password = null,
                                      $hasUserData = false, $returnOrgAndBranches = false, $useSupportOperator = null)
    {
        $suporteOperatorService = ServiceInstance::getSupportOperator();
        $isSupportOperator = false;
        $supportOperator = null;
        $userData = null;

        // condição para versão 4.4 (ou superior) do módulo de login
        if ($user && (is_null($useSupportOperator) || $useSupportOperator === true)) {
            $supportOperator = $suporteOperatorService->isSupportOperator($user, $password);
            $isSupportOperator = $supportOperator ? true : false;
            if($isSupportOperator) {
                $organizationId = $supportOperator['NRORGSUP'];
            }
            $userData = $isSupportOperator ? $suporteOperatorService->getUserData($supportOperator, $password) : null;

            if(!$isSupportOperator && !$hasUserData){
                return array (
                    "USER_DATA" => $userData,
                    "IS_SUPPORT_OPERATOR" => $isSupportOperator
                );
            }
        }

        $repositoriesInfo = array();
        $fileServerAPIKey = null;

        try {
            $repositoriesInfo = RepositoryInstance::getOmProdutoRepositorio()->getMigrationRepositoriesInfo();
        } catch(\Exception $e) {}
        $organization = RepositoryInstance::getOrganizacao()->getOrganizacaoByNrorg($organizationId);
        $isSaaSOrg    = !empty($organization) && $organization['IDORGSAAS'] != 'N';
        $primaryId    = !empty($organization) && !empty($organization['NRORGPRINCIPAL']) ? 'S' : 'P';

        if($isSaaSOrg) {
            try {
                $fileServerAPIKey = ServiceInstance::getOrganizationHandler()->getAPIKey($organizationId);
            } catch (\Exception $e) {}
        }
        $productElements = $this->getProductElements($productId, $organizationId, $repositoriesInfo, $elementIds);
        $server = $this->checkServerAndIfIsProduction($serverId);
        $organizationAndBranchesIds = $returnOrgAndBranches ? RepositoryInstance::getOrganizacao()->findOrganizationsAndTheirRespectiveBranches($organizationId) : array();
        $redirection = $this->getRedirectUrl($organizationId, $productId);

        return array(
            "VISIBLE_ELEMENTS"    => $productElements['AVAILABLE_ELEMENTS'],
            "EXPIRED_ELEMENTS"    => $productElements['EXPIRED_ELEMENTS'],
            "PRODUCTS_PERMISSION" => $productElements['PRODUCTS'],
            "REPOSITORIES_INFO"   => $productElements['PRODUCT_REPOSITORIES'],
            "RELATED_ORG_IDS"     => $productElements['RELATED_ORG_IDS'],
            "REDIRECT_URL"        => $redirection['REDIRECT_URL'],
            "REDIRECT_TO_RC"      => $redirection['REDIRECT_TO_RC'],
            "RELATED_ORGS"        => $this->findRelatedOrganizations($productElements['RELATED_ORG_IDS']),
            "VARIABLE_MEASURE"    => $server ? $this->getVariableMeasure($productId, $organizationId)   : array(),
            "GENERAL_COLLECTION"  => $server ? $this->getGeneralCollection($productId, $organizationId) : array(),
            "TOKEN"               => $this->getToken(self::LIBRARIES_CLIENT_ID),
            "PRIMARY_ID"          => $primaryId,
            "FILE_SERVER_API_KEY" => $fileServerAPIKey,
            "EXPIRATION_DATE_PER" => $this->getExpirationDatePer($productId, $organizationId),
            "USER_DATA"           => $userData,
            "IS_SUPPORT_OPERATOR" => $isSupportOperator,
            'ORGANIZATION_AND_BRANCHES_IDS' => $organizationAndBranchesIds,
            "ORG_IS_ACTIVE"       => empty($organization) ? 'S' : $organization['IDATIVO'],
        );
    }

    /**
     * @param $serverId
     * @return bool
     */
    public function checkServerAndIfIsProduction($serverId)
    {
        $configMedidaVariavelRepository = RepositoryInstance::getOmConfigMedidaVariavel();
        $server = $configMedidaVariavelRepository->findServerById($serverId);
        if(!empty($server) && $server[0]['IS_PRODUCTION'] == 'N'){
            return false;
        }
        return true;
    }

    /**
     * @param $token
     * @throws \Exception
     */
    public function logout($token) {
        try {
            ServiceInstance::getOAuth()->deleteToken($token);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getExpirationDatePer($productId, $organizationId)
    {
        $data = RepositoryInstance::getOrganizacao()->getExpirationDatePer($productId, $organizationId);
        return !empty($data) ? $data[0]['VALIDATE_PER'] : null;
    }

    public function validateVariableMeasureError($row) {
        if($row['ERROR']) {
            $error = @json_decode($row['ERROR'], true);
            if(is_array($error) && empty($error)) {
                return 'A busca dos dados a serem coletados não retornou nenhum registro.';
            }
        }
        return $row['ERROR'];
    }

    /**
     * @param $rows
     * @return bool
     * @throws \Zeedhi\Framework\DB\Mongo\Exception
     * @throws \Exception
     */
    public function saveVariableMeasure($rows, $collectType) {
        foreach ($rows as $row) {
            $initDate     = intval(Utilities::getReversedDate($row['INIT_COMPETENCE']));
            $endDate      = intval(Utilities::getReversedDate($row['END_COMPETENCE']));
            $registers    = $this->findVariableMeasure($row['ID'], $row['ORGANIZATION_ID'], $initDate, $endDate, $row['CONTRACT_ID']);
            $upInsert     = empty($registers) ? true : false;
            $row['ERROR'] = $this->validateVariableMeasureError($row);
            $rowToUpdate  = $this->getMongoRowToUpdateVariableMeasure($row, $initDate, $endDate, $upInsert, $registers);

            if ($rowToUpdate['lastStatus'] === self::VARIABLE_MEASURE_ERROR_STATUS) {
                $registersTypeSuccess  = $this->findVariableMeasure($row['ID'], $row['ORGANIZATION_ID'], $initDate, $endDate, $row['CONTRACT_ID'],
                                                                    self::VARIABLE_MEASURE_SUCCESS_STATUS);
                if (!empty($registersTypeSuccess)) {
                    $rowToUpdate['lastDateSuccess'] = $registersTypeSuccess[0]['date'];
                }
            }

            $filter = array(
                'id'             => strval($row['ID']),
                'contractId'     => strval($row['CONTRACT_ID']),
                'initCompetence' => $initDate,
                'endCompetence'  => $endDate,
            );

            GeneralInstance::getMongoDB()->update($this->getVariableMeasureCollection($row['ORGANIZATION_ID']), 
                                                  $filter, ['$set' => $rowToUpdate], $upInsert);

            if ($row['ERROR']) {
                try {
                    $this->insertCollectionError($row['ORGANIZATION_ID'], $row['PRODUCT_ID'], $row['MEASURE_CODE'], $row['ERROR'], $row['CDSERVORCA'], 
                        $row['CONTRACT_ID'], $row['INIT_COMPETENCE'], $row['END_COMPETENCE'], $rowToUpdate['lastDateSuccess'], $row, $collectType);
                } catch(\Exception $e) { };
            }
        }
        return true;
    }

    public function saveGeneralCollection($rows) {
        foreach ($rows as $row) {
            $registers   = $this->findGeneralCollection($row['ID'], $row['ORGANIZATION_ID']);
            $upInsert    = empty($registers) ? true : false;
            $rowToUpdate = $this->getMongoRowToUpdateGeneralCollection($row, $upInsert, $registers);
            $filter = array(
                'id' => strval($row['ID'])
            );
            GeneralInstance::getMongoDB()->update($this->getGeneralCollectionName($row['ORGANIZATION_ID']),
                                                  $filter, ['$set' => $rowToUpdate], $upInsert);

            if ($row['ERROR']) {
                try {
                    $this->sendErrorEmail($row['ORGANIZATION_ID'], $row['PRODUCT_ID'], $row['PRODUCT_NAME'], $row['MEASURE_NAME'], $row['ERROR']);
                } catch(\Exception $e) { };
            }
        }
        return true;
    }

    /**
     * @param $row
     * @param $initDate
     * @param $endDate
     * @param $upInsert
     * @param $registers
     * @return array
     * @throws \Exception
     */

    public function getMongoRowToUpdateVariableMeasure($row, $initDate, $endDate, $upInsert, $registers) {
        $actualDate  = new \DateTime();
        $rowToUpdate = array(
            'id'             => strval($row['ID']),
            'productId'      => strval($row['PRODUCT_ID']),
            'productName'    => $row['PRODUCT_NAME'],
            'operatorCode'   => $row['OPERATOR_CODE'],
            'operatorName'   => $row['OPERATOR_NAME'],
            'measureCode'    => strval($row['MEASURE_CODE']),
            'measureName'    => $row['MEASURE_NAME'],
            'contractRef'    => $row['CONTRACT_REFERENCE'],
            'contractId'     => strval($row['CONTRACT_ID']),
            'contractExpId'  => strval($row['CONTRACT_EXP_ID']),
            'serviceName'    => isset($row['NMSERVORCA']) ? $row['NMSERVORCA'] : null,
            'serviceId'      => isset($row['CDSERVORCA']) ? strval($row['CDSERVORCA']) : null,
            'initCompetence' => $initDate,
            'endCompetence'  => $endDate,
            'date'           => intval($actualDate->format('Ymd')),
            'lastDateSuccess' => null
        );
        if ($row['ERROR']) {
            $rowToUpdate['error']      = $row['ERROR'];
            $rowToUpdate['lastStatus'] = self::VARIABLE_MEASURE_ERROR_STATUS;
            if(!$upInsert && isset($registers[0]['count'])) {
                $rowToUpdate['log']   = $registers[0]['log'];
                $rowToUpdate['count'] = strval($registers[0]['count']);
            }
        } else {
            $rowToUpdate['log']        = $row['LOG'];
            $rowToUpdate['count']      = strval($row['COUNT']);
            $rowToUpdate['error']      = null;
            $rowToUpdate['lastStatus'] = self::VARIABLE_MEASURE_SUCCESS_STATUS;
        }
        return $rowToUpdate;
    }

    public function getMongoRowToUpdateGeneralCollection($row, $upInsert, $registers) {
        $actualDate  = new \DateTime();
        $rowToUpdate = array(
            'id'             => strval($row['ID']),
            'productId'      => strval($row['PRODUCT_ID']),
            'productName'    => $row['PRODUCT_NAME'],
            'operatorCode'   => $row['OPERATOR_CODE'],
            'operatorName'   => $row['OPERATOR_NAME'],
            'dtLimite'       => intval(Utilities::getReversedDate($row['DT_LIMITE'])),
            'date'           => intval($actualDate->format('Ymd'))
        );
        if ($row['ERROR']) {
            $rowToUpdate['error']      = $row['ERROR'];
            $rowToUpdate['lastStatus'] = self::VARIABLE_MEASURE_ERROR_STATUS;
            if(!$upInsert) {
                $rowToUpdate['log']   = $registers[0]['log'];
            }
        } else {
            $rowToUpdate['log']        = $row['LOG'];
            $rowToUpdate['error']      = null;
            $rowToUpdate['lastStatus'] = self::VARIABLE_MEASURE_SUCCESS_STATUS;
        }

        return $rowToUpdate;
    }

    /**
     * @param $clientId
     * @return string
     * @throws \Zeedhi\Framework\Security\OAuth\Exception
     */
    private function getToken($clientId){
        return ServiceInstance::getOAuth()->grantAccessToken($clientId, null, array('sessionId' => GeneralInstance::getSession()->getId()));
    }

    /**
     * @param $productId
     * @param $nrOrg
     * @param $repositoriesInfo
     * @param $elementIds
     * @throws \Exception
     * @return array
     */
    private function getProductElements($productId, $nrOrg, $repositoriesInfo, $elementIds) {
        $orgProducts     = RepositoryInstance::getOmProdutoElemento()->getProductsWithElementsByNrOrgId($nrOrg);
        $primaryOrgInfo  = $this->getPrimaryOrganizationInfo($nrOrg, $orgProducts);
        $orgProducts     = $primaryOrgInfo['ORG_PRODUCTS'];
        $relatedOrgIds   = $primaryOrgInfo['RELATED_ORG_IDS'];
        $productRepositories = $product = array();

        foreach($repositoriesInfo as $repository) {
            if ($repository['NREXTERNAL_PRODUCT_ID'] == $productId) {
                $productRepositories[] = $repository;
            }
            foreach($orgProducts as &$orgProduct) {
                if(!is_array($orgProduct['elements'])) {
                    $orgProduct['elements'] = explode(',', $orgProduct['elements']);
                }
                if(!is_array($orgProduct['expiredElements'])) {
                    $orgProduct['expiredElements'] = explode(',', $orgProduct['expiredElements']);
                }
                if(!isset($orgProduct['migrationRepositories'])) {
                    $orgProduct['migrationRepositories'] = array();
                }
                if($orgProduct['productId'] == $repository['NREXTERNAL_PRODUCT_ID']) {
                    $orgProduct['migrationRepositories'][] = $repository;
                }
                if(empty($product) && $orgProduct['productId'] == $productId) {
                    $product = $orgProduct;
                }
            }
        }

        if(empty($product)) {
            foreach ($orgProducts as $orgProduct) {
                if ($orgProduct['productId'] == $productId) {
                    if(!is_array($orgProduct['elements'])) {
                        $orgProduct['elements'] = explode(',', $orgProduct['elements']);
                    }
                    if(!is_array($orgProduct['expiredElements'])) {
                        $orgProduct['expiredElements'] = explode(',', $orgProduct['expiredElements']);
                    }
                    if(!isset($orgProduct['migrationRepositories'])) {
                        $orgProduct['migrationRepositories'] = array();
                    }
                    $product = $orgProduct;
                }
            }
        }

        $orgAllMenusVisible = Utilities::getOrgsAllMenusVisibleParameter();
        $elements = array();
        $expiredElements = isset($product['expiredElements']) ? $product['expiredElements'] : [];
        if(!empty($orgAllMenusVisible) && !empty($elementIds)) {
            $elements = $elementIds;
            $expiredElements = array();
        }

        if(empty($elements)) {
            if (empty($product)) {
                LoginException::productIsNotAvailable();
            }

            $defaultElements = RepositoryInstance::getOmProdutoElemento()->getProductDefaultElements($productId, $nrOrg);
            if(!empty($defaultElements)) {
                $defaultElements = Utilities::getArrayFromAssociativeArray($defaultElements);
                $product['elements'] = array_merge($product['elements'], $defaultElements);
            }

            if (empty($product['elements'])) {
                LoginException::menuItemsNotFound();
            }
            $elements = $product['elements'];
        }

        return array(
            'AVAILABLE_ELEMENTS'   => $elements,
            'EXPIRED_ELEMENTS'     => $expiredElements,
            'PRODUCTS'             => $orgProducts,
            'PRODUCT_REPOSITORIES' => $productRepositories,
            'RELATED_ORG_IDS'      => $relatedOrgIds
        );
    }

    /**
     * @param $nrOrg
     * @param $productId
     * @return string|null
     */
    private function getRedirectUrl($nrOrg, $productId) {
        $result = array(
            'REDIRECT_URL'   => null,
            'REDIRECT_TO_RC' => null,
        );
        $data = RepositoryInstance::getOmProdutoOrg()->getURLRedirectByNrorgAndProductId($nrOrg, $productId);
        if(!empty($data)) {
            if($data['IDUTREDIRECTRC'] == 'S') {
                $result['REDIRECT_TO_RC'] = true;
            } else if(!empty($data['DSURLREDIRECT'])) {
                $result['REDIRECT_URL'] = $data['DSURLREDIRECT'];
            }
        } 
        
        return $result;
    }

    /**
     * @param $orgIds
     * @return array
     */
    private function findRelatedOrganizations($orgIds)
    {
        return RepositoryInstance::getOrganizacao()->findRelatedOrganizations($orgIds);
    }

    /**
     * @param $productId
     * @param $organizationId
     * @return array
     * @throws \Zeedhi\Framework\DB\Mongo\Exception
     */
    private function getVariableMeasure($productId, $organizationId) {
        $variableMeasure = array();
        $configMedidaVariavelRepository = RepositoryInstance::getOmConfigMedidaVariavel();
        $contracts = $configMedidaVariavelRepository->findActualCustomerContracts(OmConfigMedidaVariavel::VARIABLE_MEASURE_COLLECT_CLIENT_TYPE,
            $productId, $organizationId);

        foreach($contracts as $contract) {
            $variableMeasureConfig = $configMedidaVariavelRepository->findVariableMeasureByExternalProductId($productId,
                $contract['CDTPMEDIDASERV'], $contract['CDSERVORCA'], $contract['NMSERVORCA'], 'PHP');

            if(empty($variableMeasureConfig)) {
                $variableMeasureConfig = $configMedidaVariavelRepository->findVariableMeasureRelatedOrg($contract['CDTPMEDIDASERV'],
                    $contract['CDSERVORCA'], $contract['NREXTERNAL_PRODUCT_ID'], $contract['NMPRODUTO'], 'PHP');
            }

            foreach($variableMeasureConfig as $variableMeasureRegister) {
                $varMeasureToSave = $this->validateVariableMeasureRegister($variableMeasureRegister, $contract, $organizationId);
                if($varMeasureToSave) {
                    $variableMeasure[] = $varMeasureToSave;
                }
            }
        }
        return $variableMeasure;
    }

    /**
     * @param $lastDate
     * @param $recurrenceDays
     * @param string $contractEndDate
     * @return bool
     * @throws \Exception
     */
    private function validateVariableMeasureAndGeneralCollectionDate($lastDate, $recurrenceDays, $contractEndDate) {
        $actualDate    = new \DateTime();
        $actualDateStr = $actualDate->format('d/m/Y');
        $actualDateInt = intval($actualDate->format('Ymd'));
        return (($lastDate + intval($recurrenceDays)) - $actualDateInt <= 0) || ($actualDateStr == $contractEndDate);
    }

    /**
     * @param $id
     * @param $organizationId
     * @param $initDate
     * @param $endDate
     * @param $contractId
     * @return array
     * @throws \Zeedhi\Framework\DB\Mongo\Exception
     */
    private function findVariableMeasure($id, $organizationId, $initDate, $endDate, $contractId, $lastStatus = null) {
        $filter = array(
            'id'             => strval($id),
            'contractId'     => strval($contractId),
            'initCompetence' => $initDate,
            'endCompetence'  => $endDate,
        );

        if ($lastStatus) {
            $filter['lastStatus'] = $lastStatus;
        }

        return GeneralInstance::getMongoDB()->find($this->getVariableMeasureCollection($organizationId), $filter);
    }

    private function findGeneralCollection($id, $organizationId) {
        $filter = array(
            'id' => strval($id)
        );
        return GeneralInstance::getMongoDB()->find($this->getGeneralCollectionName($organizationId), $filter);
    }

    /**
     * @param $organizationId
     * @return string
     */
    private function getVariableMeasureCollection($organizationId) {
        return self::VARIABLE_MEASURE_COLLECTION . '_' . $organizationId;
    }

    private function getGeneralCollectionName($organizationId) {
        return self::GENERAL_COLLECTION . '_' . $organizationId;
    }

    /**
     * @param $productId
     * @param $organizationId
     * @return array
     * @throws \Exception
     */
    private function getGeneralCollection($productId, $organizationId) {
        $variableMeasure = array();
        $configMedidaVariavelRepository = RepositoryInstance::getOmConfigMedidaVariavel();

        $omConfigCellections = $configMedidaVariavelRepository->findConfigCollect($productId, $organizationId);

        foreach($omConfigCellections as $configCollect) {
            $varMeasureToSave = $this->validateGeneralCollectionRegister($configCollect, $organizationId);
            if($varMeasureToSave) {
                $variableMeasure[] = $varMeasureToSave;
            }
        }

        return $variableMeasure;
    }


    /**
     * @param $organizationId
     * @param $productId
     * @param $productName
     * @param $measureName
     * @param $error
     * @param $serviceName
     */
    private function sendErrorEmail($organizationId, $productId, $productName, $measureName, $error,
                                    $serviceName = '', $serviceCode = '', $contractId = '') {
        $emails = Utilities::getVariableMeasureEmailToSendParameter();
        $serverName = Utilities::getServerNameParameter();
        foreach(explode(';', $emails) as $email) {
            $body = "<h3>Ocorreu um erro na rotina de buscar medidas variáveis. Dados abaixo:</h3>
                 <strong>- Servidor do License:</strong> $serverName <br/>
                 <strong>- Número da Organização:</strong> $organizationId <br/>
                 <strong>- ID do Produto (DevOps):</strong> $productId <br/>
                 <strong>- Nome do Produto:</strong> $productName <br/>
                 <strong>- Medida Variável:</strong> $measureName <br/>
                 <strong>- Número do contrato:</strong> $contractId <br/>
                 <strong>- Código do Serviço:</strong> $serviceCode <br/>
                 <strong>- Nome do Serviço:</strong> $serviceName <br/><br/>
                 <strong>--> Erro:</strong> $error";

            $mimePart = new ZendMimePart(utf8_decode($body));
            $mimePart->type = 'text/html';

            $mimeMessage = new ZendMimeMessage();
            $mimeMessage->setParts(array($mimePart));
            $body = $mimeMessage;

            $smtpMail = GeneralInstance::getSmtpMail();
            $smtpMail->setSubject(self::VARIABLE_MEASURE_BASE_SUBJECT_EMAIL);
            $smtpMail->setBody($body);
            $smtpMail->setFrom(Utilities::getDefaultEmailParameter());
            $smtpMail->setTo($email);
            $smtpMail->send();
        }
    }

    private function insertCollectionError($organizationId, $productId, $measureCode, $error, $serviceCode, $contractId, 
                                           $iniCompetence, $endCompetence, $lastDateSuccess, $row, $collectType)
    {
        $operatorCode  = "000000000997";
        $server        = Utilities::getServerNameParameter();
        $productId     = $productId == '0' || empty($productId) ? 'NULL' : $productId;
        $serviceCode   = empty($serviceCode) ? 'NULL' : $serviceCode;
        $id            = GeneralInstance::getNewCode()->getCodeByTableName('ERROS_COLETA', 20, 1, 10, true);
        $iniCompetence = "TRUNC(TO_DATE('$iniCompetence', 'DD/MM/YYYY'))";
        $endCompetence = "TRUNC(TO_DATE('$endCompetence', 'DD/MM/YYYY'))";
        
        if($lastDateSuccess) {
            $lastDateSuccess = \DateTime::createFromFormat('Ymd', $lastDateSuccess)->format('d/m/Y');
            $lastDateSuccess = "TRUNC(TO_DATE('$lastDateSuccess', 'DD/MM/YYYY'))";
        } else {
            $lastDateSuccess = "NULL";
        }

        $scriptCount = !empty($row['SCRIPT_COUNT'])    ? $row['SCRIPT_COUNT']    : null;
        $scriptLog   = !empty($row['SCRIPT_LOG'])      ? $row['SCRIPT_LOG']      : null;
        $idTpScript  = !empty($row['IDTPSCRIPT'])      ? $row['IDTPSCRIPT']      : null;
        $strConexao  = !empty($row['DSSTRINGCONEXAO']) ? $row['DSSTRINGCONEXAO'] : null;
        $idWallet    = !empty($row['IDUTWALLET'])      ? $row['IDUTWALLET']      : null;
        $idColetor   = !empty($row['IDCOLETOR'])       ? $row['IDCOLETOR']       : null;    

        $query = "
            INSERT INTO ERROS_COLETA (ID, SERVIDOR, NRORG_ID, CDTPMEDIDASERV, NRSEQCONTRATO, CDSERVORCA, PRODUCT_ID, 
                                      ERRO_COLETA, CDOPERINCLUSAO, CDOPERULTATU, DTINICOMPET, DTFINCOMPET, DTULTCOLETASUC,
                                      SCRIPT_COUNT, SCRIPT_LOG, IDTPSCRIPT, DSSTRINGCONEXAO, IDUTWALLET, IDCOLETOR, IDTPCOLETA)
                              VALUES ($id, '$server', $organizationId, $measureCode, $contractId, $serviceCode, $productId, 
                                      :ERRO_COLETA, '$operatorCode', '$operatorCode', $iniCompetence, $endCompetence, $lastDateSuccess,
                                      :SCRIPT_COUNT, :SCRIPT_LOG, :IDTPSCRIPT, :DSSTRINGCONEXAO, :IDUTWALLET, :IDCOLETOR, :IDTPCOLETA)
        ";
        $params = array(
            'ERRO_COLETA'     => $error,
            'SCRIPT_COUNT'    => $scriptCount,
            'SCRIPT_LOG'      => $scriptLog,
            'IDTPSCRIPT'      => $idTpScript,
            'DSSTRINGCONEXAO' => $strConexao,
            'IDUTWALLET'      => $idWallet,
            'IDCOLETOR'       => $idColetor,
            'IDTPCOLETA'      => $collectType
        );

        GeneralInstance::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    /**
     * @param $contract
     * @param $variableMeasureRegister
     */
    private function setCustomerContractParameters($contract, &$variableMeasureRegister) {
        $variableMeasureRegister['DTINICOMPET']      = $contract['DTINICOMPET'];
        $variableMeasureRegister['DTFINCOMPET']      = $contract['DTFINCOMPET'];
        $variableMeasureRegister['DSREFERENCIAORCA'] = $contract['DSREFERENCIAORCA'];
        $variableMeasureRegister['NRSEQCONTRATO']    = $contract['NRSEQCONTRATO'];
        $variableMeasureRegister['NRVENCPREVCONT']   = $contract['NRVENCPREVCONT'];
    }

    /**
     * @param bool $isTypeClient
     * @throws \Zeedhi\Framework\DB\Mongo\Exception
     */
    public function collectVariableMeasure($isTypeClient) {
        $configMedidaVariavelRepository = RepositoryInstance::getOmConfigMedidaVariavel();
        $collectType = $isTypeClient ? OmConfigMedidaVariavel::VARIABLE_MEASURE_COLLECT_CLIENT_TYPE : OmConfigMedidaVariavel::VARIABLE_MEASURE_COLLECT_TEKNISA_TYPE;
        $contracts = $configMedidaVariavelRepository->findActualCustomerContracts($collectType);
        foreach($contracts as $contract) {
            $variableMeasureConfig = array();
            $scriptType = $isTypeClient ? 'SQL' : null;
            echo "### Contrato \"" . $contract['NRSEQCONTRATO'] . "\" (" . $contract['DTINICOMPET'] . " - " . $contract['DTFINCOMPET'] . ") Cliente \"" . $contract['NMFANTCLIE'] . "\" (\"" . $contract['NMRAZSOCCLIE'] . "\")\n";
            if($contract['NREXTERNAL_PRODUCT_ID']) {
                $variableMeasureConfig = $configMedidaVariavelRepository->findVariableMeasureByExternalProductId($contract['NREXTERNAL_PRODUCT_ID'], $contract['CDTPMEDIDASERV'],
                    '', '', $scriptType);
                echo "Qt. config. produto \"" . $contract['NMPRODUTO'] . "\": " . count($variableMeasureConfig) . "\n";
            } else if(empty($variableMeasureConfig) && !empty($contract['CDSERVORCA'])) {
                $variableMeasureConfig = $configMedidaVariavelRepository->findVariableMeasureRelatedOrg($contract['CDTPMEDIDASERV'], $contract['CDSERVORCA'],
                    $contract['NREXTERNAL_PRODUCT_ID'], $contract['NMPRODUTO'], $scriptType);
                echo "Qt. config. medida \"" . $contract['DSTPMEDIDASERV'] . "\" e o serviço \"" . $contract['NMSERVORCA'] . "\": " . count($variableMeasureConfig) . "\n";
            }

            foreach($variableMeasureConfig as $variableMeasureRegister) {
                $varMeasureToSave = $this->validateVariableMeasureRegister($variableMeasureRegister, $contract, $contract['NRORG']);
                $isExecuteScript = empty($varMeasureToSave) ? 'NÃO' : 'SIM';
                echo "Executar os scripts da medida \"" . $variableMeasureRegister['DSTPMEDIDASERV'] . "\": " . $isExecuteScript;

                if($varMeasureToSave) {
                    if($variableMeasureRegister['IDTPSCRIPT'] == 'PHP') {
                        if($collectType == OmConfigMedidaVariavel::VARIABLE_MEASURE_COLLECT_TEKNISA_TYPE) {
                            echo "\n--> PHP";
                            $varMeasureScript = $this->callPHPScriptVariableMeasure($varMeasureToSave, $variableMeasureRegister['NREXTERNAL_PRODUCT_ID'],
                                '000000000997', 'Teknisa', $contract, $variableMeasureRegister);
                        } else {
                            $varMeasureScript = null;
                            echo "\nScripts do tipo \"PHP\" não serão executados para medidas variáveis do tipo \"CLIENTE\".";
                        }
                    } else {
                        echo "\n--> SQL";
                        $varMeasureScript = $this->callSQLScriptVariableMeasure($varMeasureToSave, $variableMeasureRegister['NREXTERNAL_PRODUCT_ID'],
                            '000000000997', 'Teknisa', $contract, $variableMeasureRegister);
                    }
                    if($varMeasureScript) {
                        echo "\nSALVANDO OS REGISTROS DA COLETA...";
                        $this->saveVariableMeasure(array($varMeasureScript), $collectType);
                    } else {
                        echo "\nNÃO FORAM DETECTADOS DADOS DE COLETA PARA GRAVAÇÃO.";
                    }
                }
            }
            echo "\n\n";
        }
        echo "<<< Processo finalizado com sucesso!\n";
    }

    /**
     * @param $variableMeasureRegister
     * @param $contract
     * @param $organizationId
     * @return null
     * @throws \Zeedhi\Framework\DB\Mongo\Exception
     * @throws \Exception
     */
    private function validateVariableMeasureRegister($variableMeasureRegister, $contract, $organizationId) {
        $variableMeasure = null;
        $variableMeasureRegister['ACTUAL_COUNT'] = null;
        $this->setCustomerContractParameters($contract, $variableMeasureRegister);
        $initDate  = intval(Utilities::getReversedDate($contract['DTINICOMPET']));
        $endDate   = intval(Utilities::getReversedDate($contract['DTFINCOMPET']));
        $registers = $this->findVariableMeasure($variableMeasureRegister['ID'], $organizationId, $initDate, $endDate, $contract['NRSEQCONTRATO']);
        $register  = empty($registers) ? null : $registers[0];

        if (empty($register)) {
            $variableMeasure = $variableMeasureRegister;
        } else {
            $lastDate       = isset($register['date'])       ? $register['date']       : null;
            $lastStatus     = isset($register['lastStatus']) ? $register['lastStatus'] : null;
            $recurrenceDays = $variableMeasureRegister['QTD_DIAS_REAPLICAR'];
            if ($lastStatus && $lastStatus == self::VARIABLE_MEASURE_ERROR_STATUS) {
                $recurrenceDays = 1;
            }
            if ($lastDate && $this->validateVariableMeasureAndGeneralCollectionDate($lastDate, $recurrenceDays, $contract['DTFINCOMPET'])) {
                $variableMeasureRegister['ACTUAL_DATE'] = $lastDate;
                if ($lastStatus && $lastStatus == self::VARIABLE_MEASURE_ERROR_STATUS) {
                    $variableMeasureRegister['ACTUAL_COUNT'] = null;
                } else {
                    $variableMeasureRegister['ACTUAL_COUNT'] = isset($register['count']) ? $register['count'] : null;
                }
                $variableMeasure = $variableMeasureRegister;
            }
        }
        return $variableMeasure;
    }

    /**
     * @param $configCollect
     * @param $organizationId
     * @return null
     * @throws \Exception
     */
    private function validateGeneralCollectionRegister($configCollect, $organizationId) {
        $variableMeasure = null;
        $configCollect['ACTUAL_COUNT'] = null;
        $registers = $this->findGeneralCollection($configCollect['ID'], $organizationId);
        $register  = empty($registers) ? null : $registers[0];

        if (empty($register)) {
            $variableMeasure = $configCollect;
        } else {
            $lastDate       = isset($register['date'])       ? $register['date']       : null;
            $lastStatus     = isset($register['lastStatus']) ? $register['lastStatus'] : null;
            $recurrenceDays = $configCollect['QTD_DIAS_REAPLICAR'];
            if ($lastStatus && $lastStatus == self::VARIABLE_MEASURE_ERROR_STATUS) {
                $recurrenceDays = 1;
            }
            if ($lastDate && $this->validateVariableMeasureAndGeneralCollectionDate($lastDate, $recurrenceDays, $configCollect['DT_LIMITE'])) {
                $configCollect['ACTUAL_DATE'] = $lastDate;
                if ($lastStatus && $lastStatus == self::VARIABLE_MEASURE_ERROR_STATUS) {
                    $configCollect['ACTUAL_COUNT'] = null;
                } else {
                    $configCollect['ACTUAL_COUNT'] = isset($register['count']) ? $register['count'] : null;
                }
                $variableMeasure = $configCollect;
            }
        }
        return $variableMeasure;
    }

    /**
     * @param $varMeasureToSave
     * @param $product
     * @param $userId
     * @param $userName
     * @param $contract
     * @param $variableMeasureRegister
     * @return array|null
     */
    private function callPHPScriptVariableMeasure($varMeasureToSave, $product, $userId, $userName, $contract, $variableMeasureRegister) {
        $organizationId = $contract['NRORG'];
        $varMeasureToSave['CDSERVORCA'] = isset($contract['CDSERVORCA']) ? $contract['CDSERVORCA'] : '';
        $varMeasureToSave['IDPRIMARIA'] = isset($contract['IDPRIMARIA']) ? $contract['IDPRIMARIA'] : 'P';
        $variableMeasureToUpdate = $count = $log = $error = $countFloat = $actualCountFloat = null;
        $countOutput = $this->execPHPScriptVariableMeasure($varMeasureToSave, $organizationId, false, $product);
        $countOutputDecoded = @json_decode($countOutput, true);
        if(empty($countOutputDecoded) && !is_array($countOutputDecoded)) {
            $error = $countOutput;
        } else {
            $count = isset($countOutputDecoded[0]) && is_array($countOutputDecoded[0]) ?
                $countOutputDecoded[0]['COUNT'] : $countOutputDecoded['COUNT'];
        }
        if($count) {
            $countFloat = floatval(str_replace(",", ".", str_replace(".", "", $count)));
        }
        if($varMeasureToSave['ACTUAL_COUNT']) {
            $actualCountFloat = floatval(str_replace(",", ".", str_replace(".", "", $varMeasureToSave['ACTUAL_COUNT'])));
        }
        if(!empty($error)) {
            $variableMeasureToUpdate = $this->buildVariableMeasureRow($varMeasureToSave, $count, $log, $product,
                $userId, $userName, $organizationId, $error, $variableMeasureRegister);
        } else if(empty($varMeasureToSave['ACTUAL_COUNT']) || $actualCountFloat < $countFloat) {
            $logOutput = $this->execPHPScriptVariableMeasure($varMeasureToSave, $organizationId, true, $product);
            $logOutputDecoded = @json_decode($logOutput, true);
            if(empty($logOutputDecoded) && !is_array($logOutputDecoded)) {
                $error = $logOutput;
            } else {
                $log = $logOutputDecoded;
            }
            $variableMeasureToUpdate = $this->buildVariableMeasureRow($varMeasureToSave, $count, $log, $product,
                $userId, $userName, $organizationId, $error, $variableMeasureRegister);
        }
        return $variableMeasureToUpdate;
    }

    /**
     * @param $variableMeasureRegister
     * @param $organizationId
     * @param $isLogScript
     * @param $devOpsId
     * @return string
     */
    private function execPHPScriptVariableMeasure($variableMeasureRegister, $organizationId, $isLogScript, $devOpsId) {
        $scriptContents   = $isLogScript ? $variableMeasureRegister['PHP_SCRIPT_LOG'] : $variableMeasureRegister['PHP_SCRIPT_COUNT'];
        $contractInitDate = $variableMeasureRegister['DTINICOMPET'];
        $contractEndDate  = $variableMeasureRegister['DTFINCOMPET'];
        $contractId       = $variableMeasureRegister['NRSEQCONTRATO'];
        $omProductId      = $variableMeasureRegister['NRPRODUTO_ID'];
        $serviceId        = $variableMeasureRegister['CDSERVORCA'];
        $primaryId        = $variableMeasureRegister['IDPRIMARIA'];
        $connectionParams = GeneralInstance::getEntityManager()->getConnection()->getParams();
        $connectionParams = json_encode($connectionParams);

        $fileName = $isLogScript ? "/phpFileLog.php" : "/phpFileCount.php";
        $fileName = Utilities::getTmpPathParameter() . $fileName;
        file_put_contents($fileName, $scriptContents);
        $params = " '$connectionParams' '$organizationId' '$contractInitDate' '$contractEndDate' '$contractId' '$omProductId' '$serviceId' '$devOpsId' '$primaryId' ";
        $output = trim(shell_exec("php " . $fileName . $params . "2>&1"));
        unlink($fileName);
        return $output;
    }


    /**
     * @param $varMeasureToSave
     * @param $product
     * @param $userId
     * @param $userName
     * @param $contract
     * @param $variableMeasureRegister
     * @return array|null
     */
    private function callSQLScriptVariableMeasure($varMeasureToSave, $product, $userId, $userName, $contract, $variableMeasureRegister) {
        $organizationId = $contract['NRORG'];
        $mainOrganizationId = empty($contract['NRORGPRINCIPAL']) ? $contract['NRORG'] : $contract['NRORGPRINCIPAL'];
        $varMeasureToSave['CDSERVORCA'] = isset($contract['CDSERVORCA']) ? $contract['CDSERVORCA'] : '';
        $activeFilter = " AND IDATIVO = 'S'";

        $productOrg = RepositoryInstance::getOmProdutoOrg()->findOneBy(array('nrprodutoId' => $variableMeasureRegister['NRPRODUTO_ID'], 'nrorgId' => $mainOrganizationId));
        if(empty($productOrg) && $organizationId != $mainOrganizationId) {
            $productOrg = RepositoryInstance::getOmProdutoOrg()->findOneBy(array('nrprodutoId' => $variableMeasureRegister['NRPRODUTO_ID'], 'nrorgId' => $organizationId));
        }

        //Parâmetros de coletor relacionados com o Produto da Organização
        $parameters = empty($productOrg) ? null : RepositoryInstance::getOmProdutoOrgColetor()->getProductOrgCollector($productOrg->getId(), $activeFilter);
        $parameters = empty($parameters) ? null : $parameters[0];
        if(empty($parameters)) {
            //Parâmetros de coletor relacionados diretamente com o Produto
            $parameters = RepositoryInstance::getOmProdutoColetor()->getProductCollector($variableMeasureRegister['NRPRODUTO_ID'], $activeFilter);
            $parameters = empty($parameters) ? null : $parameters[0];
            if(empty($parameters)) {
                //Parâmetros de coletor relacionados com a Organização
                $parameters = $this->validateOrgCollectorParams($mainOrganizationId);
                if(empty($parameters)) {
                    $filter  = " AND OPD.IDAMBORG = '" . $contract['IDORGSAAS'] . "' ";
                    $filter .= " AND OPD.IDATIVO = 'S' ";
                    //Parâmetros padrões de coletor por Hospedagem
                    $parameters = RepositoryInstance::getOmParamOrgDefault()->getDefaultParameters($filter);
                    $parameters = empty($parameters) ? null : $parameters[0];
                }
            }
        }

        $variableMeasureToUpdate = $count = $log = $error = $countFloat = $actualCountFloat = null;
        if($contract['IDORGSAAS'] != 'N' || ($contract['IDORGSAAS'] == 'N' && !empty($parameters))) {
            $varMeasureToSave['SQL_SCRIPT_COUNT'] = $varMeasureToSave['PHP_SCRIPT_COUNT'];
            $varMeasureToSave['SQL_SCRIPT_LOG']   = $varMeasureToSave['PHP_SCRIPT_LOG'];
            if (empty($parameters)) {
                $error = 'Parametrizações de Coletor e Banco de Dados não cadastradas ou incompletas.';
                $variableMeasureToUpdate = $this->buildVariableMeasureRow($varMeasureToSave, $count, $log, $product,
                    $userId, $userName, $organizationId, $error, $variableMeasureRegister);
            } else {
                $countOutput = $this->execSQLScriptVariableMeasure($varMeasureToSave, $organizationId, false, $parameters, $contract['IDPRIMARIA'], $product, $contract['NRORGPRINCIPAL']);
                if ($countOutput['IS_ERROR']) {
                    $error = $countOutput['RESULT'];
                } else {
                    $count = $countOutput['RESULT'][0]['COUNT'];
                }
                if($count) {
                    $countFloat = floatval(str_replace(",", ".", str_replace(".", "", $count)));
                }
                if($varMeasureToSave['ACTUAL_COUNT']) {
                    $actualCountFloat = floatval(str_replace(",", ".", str_replace(".", "", $varMeasureToSave['ACTUAL_COUNT'])));
                }
                if (!empty($error)) {
                    $variableMeasureToUpdate = $this->buildVariableMeasureRow($varMeasureToSave, $count, $log, $product,
                        $userId, $userName, $organizationId, $error, $variableMeasureRegister);
                } else if (empty($varMeasureToSave['ACTUAL_COUNT']) || $actualCountFloat < $countFloat) {
                    $logOutput = $this->execSQLScriptVariableMeasure($varMeasureToSave, $organizationId, true, $parameters, $contract['IDPRIMARIA'], $product, $contract['NRORGPRINCIPAL']);
                    if ($logOutput['IS_ERROR']) {
                        $error = $logOutput['RESULT'];
                    } else {
                        $log = $logOutput['RESULT'];
                    }
                    $variableMeasureToUpdate = $this->buildVariableMeasureRow($varMeasureToSave, $count, $log, $product,
                        $userId, $userName, $organizationId, $error, $variableMeasureRegister);
                }
            }
        } else {
            echo "\nScript não executado. Cliente local sem parametrizações de Coletor e Banco de Dados.";
        }
        return $variableMeasureToUpdate;
    }

    /**
     * @param $variableMeasureRegister
     * @param $organizationId
     * @param $isLogScript
     * @param $parameters
     * @param $idPrimaria
     * @param $devOpsId
     * @return array
     */
    private function execSQLScriptVariableMeasure(&$variableMeasureRegister, $organizationId, $isLogScript, $parameters, $idPrimaria, $devOpsId, $mainOrganizationId) {
        $secScriptOrganizationId = empty($mainOrganizationId) ? 'NULL' : $organizationId;
        $secFilialOrganizationId = $organizationId;
        $organizationId = empty($mainOrganizationId) ? $organizationId : $mainOrganizationId;
        $orgsAndBranches = RepositoryInstance::getOrganizacao()->findOrganizationsAndTheirRespectiveBranches($organizationId);
        $listaLuso1 = $listaLuso2 = $listaLuso3 = $listaLuso4 = $listaLuso5 = $listaLuso6 = $listaLuso7 = $listaLuso8 = $listaLuso9 = $listaLuso10 = '_';
        foreach($orgsAndBranches as $idx => $branches) {
            if($branches['NRORG'] == $secFilialOrganizationId) {
                if($idx < 700) {
                    $listaLuso1 .= $branches['FILIAL'] . '_';
                } else if($idx >= 700 && $idx < 1400) {
                    $listaLuso2 .= $branches['FILIAL'] . '_';
                } else if($idx >= 1400 && $idx < 2100) {
                    $listaLuso3 .= $branches['FILIAL'] . '_';
                } else if($idx >= 2100 && $idx < 2800) {
                    $listaLuso4 .= $branches['FILIAL'] . '_';
                } else if($idx >= 2800 && $idx < 3500) {
                    $listaLuso5 .= $branches['FILIAL'] . '_';
                } else if($idx >= 3500 && $idx < 4200) {
                    $listaLuso6 .= $branches['FILIAL'] . '_';
                } else if($idx >= 4200 && $idx < 4900) {
                    $listaLuso7 .= $branches['FILIAL'] . '_';
                } else if($idx >= 4900 && $idx < 5600) {
                    $listaLuso8 .= $branches['FILIAL'] . '_';
                } else if($idx >= 5600 && $idx < 6300) {
                    $listaLuso9 .= $branches['FILIAL'] . '_';
                } else if($idx >= 6300 && $idx < 7000) {
                    $listaLuso10 .= $branches['FILIAL'] . '_';
                }
            }
        }

        $scriptContents = $isLogScript ? $variableMeasureRegister['PHP_SCRIPT_LOG'] : $variableMeasureRegister['PHP_SCRIPT_COUNT'];
        $scriptContents = str_replace(array(':P_NRORGSECUNDARIA', ':P_NRSEQCONTRATO', ':P_NRPRODUTO_ID',
                                            ':P_DEVOPS_ID',       ':P_DTINICOMPET',   ':P_DTFINCOMPET',
                                            ':P_IDPRIMARIA',      ':P_CDSERVORCA',    ':P_NRORG',
                                            ':P_LISTALUSO1',      ':P_LISTALUSO2',    ':P_LISTALUSO3', ':P_LISTALUSO4', ':P_LISTALUSO5',
                                            ':P_LISTALUSO6',      ':P_LISTALUSO7',    ':P_LISTALUSO8', ':P_LISTALUSO9', ':P_LISTALUSO10'
                                      ),
                                      array($secScriptOrganizationId, $variableMeasureRegister['NRSEQCONTRATO'], $variableMeasureRegister['NRPRODUTO_ID'],
                                            $devOpsId, $variableMeasureRegister['DTINICOMPET'], $variableMeasureRegister['DTFINCOMPET'],
                                            $idPrimaria, $variableMeasureRegister['CDSERVORCA'], $organizationId,
                                            $listaLuso1, $listaLuso2, $listaLuso3, $listaLuso4, $listaLuso5,
                                            $listaLuso6, $listaLuso7, $listaLuso8, $listaLuso9, $listaLuso10
                                      ),
                                      $scriptContents);

        $isWallet = $parameters['IDUTWALLET'] == 'S';
        $parameters['DSSTRINGCONEXAO'] = str_replace('[NRORG]', $organizationId, $parameters['DSSTRINGCONEXAO']);
        $parameters['DSUSUARIOBANCO']  = str_replace('[NRORG]', $organizationId, $parameters['DSUSUARIOBANCO']);

        $collectorParams = array(
            'dbUrlSsl'        => OpensslCrypt::encrypt($parameters['DSSTRINGCONEXAO']),
            'userNameSsl'     => $isWallet ? null : OpensslCrypt::encrypt($parameters['DSUSUARIOBANCO']),
            'password'        => $isWallet ? null : $parameters['DSSENHABANCO'],
            'passwordSsl'     => $isWallet ? null : $parameters['DSSENHABANCO'],
            'query'           => $scriptContents,
            'driverClass'     => self::getCollectorDriverClass($parameters['IDTPBANCO']),
            'dialect'         => self::getCollectorDialect($parameters['IDTPBANCO']),
            'oracleWalletUse' => $parameters['IDUTWALLET'] == 'S',
            'room'            => $parameters['IDCOLETOR']
        );

        $variableMeasureRegister['SQL_DSSTRINGCONEXAO'] = $parameters['DSSTRINGCONEXAO'];
        $variableMeasureRegister['SQL_IDUTWALLET']      = $parameters['IDUTWALLET'];
        $variableMeasureRegister['SQL_IDCOLETOR']       = $parameters['IDCOLETOR'];
        if($isLogScript) {
            $variableMeasureRegister['SQL_SCRIPT_LOG'] = $scriptContents;
        } else {
            $variableMeasureRegister['SQL_SCRIPT_COUNT'] = $scriptContents;
        }

        $attempts = 1;
        return $this->collectorExecuteQuery($collectorParams, $attempts);
    }

    private function collectorExecuteQuery($collectorParams, $attempts) {
        try {
            $result = GeneralInstance::getCollector()->executeQuery($collectorParams, 120);
        } catch (\Exception $e) {
            $message = $e->getMessage();
            if($message == \Util\SocketClient::NO_CLIENTS_CONNECTED) {
                if($attempts < 3) {
                    $attempts++;
                    return $this->collectorExecuteQuery($collectorParams, $attempts);
                }
                $message = 'Não foi possível conectar com o coletor "' . $collectorParams['room'] . '"';
            } 
            $result = array (
                'IS_ERROR' => true,
                'RESULT'   => $message
            );
        }
        return $result;
    }

    private function getCollectorDialect($dataBase) {
        if($dataBase == 'ORACLE') {
            return 'org.hibernate.dialect.OracleDialect';
        }
        if($dataBase == 'MY_SQL') {
            return 'org.hibernate.dialect.MySQLDialect';
        }
        if($dataBase == 'SQL_SERVER_2000') {
            return 'org.hibernate.dialect.SQLServerDialect';
        }
        if($dataBase == 'SQL_SERVER_2005') {
            return 'org.hibernate.dialect.SQLServer2005Dialect';
        }
        if($dataBase == 'SQL_SERVER_2008') {
            return 'org.hibernate.dialect.SQLServer2008Dialect';
        }
    }

    private function getCollectorDriverClass($dataBase) {
        if($dataBase == 'ORACLE') {
            return 'oracle.jdbc.driver.OracleDriver';
        }
        if($dataBase == 'MY_SQL') {
            return 'com.mysql.cj.jdbc.Driver';
        }
        if($dataBase == 'SQL_SERVER_2000') {
            return 'com.microsoft.sqlserver.jdbc.SQLServerDriver';
        }
        if($dataBase == 'SQL_SERVER_2005') {
            return 'com.microsoft.sqlserver.jdbc.SQLServerDriver';
        }
        if($dataBase == 'SQL_SERVER_2008') {
            return 'com.microsoft.sqlserver.jdbc.SQLServerDriver';
        }
    }

    /**
     * @param $organizationId
     * @return |null
     */
    private function validateOrgCollectorParams($organizationId) {
        $collectorParams = null;
        $parameters = RepositoryInstance::getOmParamOrg()->getOrganizationParameters($organizationId);
        $parameters = empty($parameters) ? null : $parameters[0];

        if($parameters && !empty($parameters['IDCOLETOR']) && !empty($parameters['DSSTRINGCONEXAO'])) {
            if($parameters['IDUTWALLET'] == 'N') {
                if(!empty($parameters['DSUSUARIOBANCO']) && !empty($parameters['DSSENHABANCO'])) {
                    $collectorParams = $parameters;
                }
            } else {
                $collectorParams = $parameters;
            }
        }

        return $collectorParams;
    }

    /**
     * @param $varMeasureToSave
     * @param $count
     * @param $log
     * @param $product
     * @param $userId
     * @param $userName
     * @param $organizationId
     * @param $error
     * @param $variableMeasureRegister
     * @return array
     */
    private function buildVariableMeasureRow($varMeasureToSave, $count, $log, $product, $userId, $userName, $organizationId, $error, $variableMeasureRegister) {
        if($varMeasureToSave['IDTPSCRIPT'] == 'PHP') {
            $connectionParams = GeneralInstance::getEntityManager()->getConnection()->getParams();
            $strConnection    = empty($connectionParams['sessionMode']) ? $connectionParams['user'] : $connectionParams['dbname'];
            $isWallet         = empty($connectionParams['sessionMode']) ? 'N' : 'S';
            $collectorId      = '';
        } else {
            $strConnection = isset($varMeasureToSave['SQL_DSSTRINGCONEXAO']) ? $varMeasureToSave['SQL_DSSTRINGCONEXAO'] : '';
            $isWallet      = isset($varMeasureToSave['SQL_IDUTWALLET'])      ? $varMeasureToSave['SQL_IDUTWALLET']      : '';
            $collectorId   = isset($varMeasureToSave['SQL_IDCOLETOR'])       ? $varMeasureToSave['SQL_IDCOLETOR']       : '';
        }
        
        return array(
            'COUNT' => $count,
            'ERROR' => $error,
            'LOG'   => $log,

            'PRODUCT_ID'   	  => $product,
            'OPERATOR_CODE'   => $userId,
            'OPERATOR_NAME'   => $userName,
            'ORGANIZATION_ID' => $organizationId,

            'ID'              => $varMeasureToSave['ID'],
            'PRODUCT_NAME' 	  => $varMeasureToSave['NMPRODUTO'],
            'MEASURE_CODE' 	  => $varMeasureToSave['CDTPMEDIDASERV'],
            'MEASURE_NAME' 	  => $varMeasureToSave['DSTPMEDIDASERV'],
            'INIT_COMPETENCE' => $varMeasureToSave['DTINICOMPET'],
            'END_COMPETENCE'  => $varMeasureToSave['DTFINCOMPET'],
            
            'SCRIPT_COUNT'    => $varMeasureToSave['IDTPSCRIPT'] == 'PHP' ? $varMeasureToSave['PHP_SCRIPT_COUNT'] : $varMeasureToSave['SQL_SCRIPT_COUNT'],
            'SCRIPT_LOG'      => $varMeasureToSave['IDTPSCRIPT'] == 'PHP' ? $varMeasureToSave['PHP_SCRIPT_LOG']   : $varMeasureToSave['SQL_SCRIPT_LOG'],
            'IDTPSCRIPT'      => $varMeasureToSave['IDTPSCRIPT'],
            'DSSTRINGCONEXAO' => $strConnection,
            'IDUTWALLET'      => $isWallet,
            'IDCOLETOR'       => $collectorId,

            'CONTRACT_REFERENCE' => $varMeasureToSave['DSREFERENCIAORCA'],
            'CONTRACT_ID'        => $varMeasureToSave['NRSEQCONTRATO'],
            'CONTRACT_EXP_ID'    => $varMeasureToSave['NRVENCPREVCONT'],

            'CDSERVORCA'    => $variableMeasureRegister['CDSERVORCA'],
            'NMSERVORCA'    => $variableMeasureRegister['NMSERVORCA']
        );
    }

    /**
     * @throws \Zeedhi\Framework\DB\Mongo\Exception
     */
    public function testConnection() {
        RepositoryInstance::getOmProdutoElemento()->getProductsWithElementsByNrOrgId(null);
        RepositoryInstance::getOrganizacao()->getOrganizacaoByNrorg(null);
        $this->findVariableMeasure(null, 20, null, null, null);
    }

    /**
     * @param $nrOrg
     * @param $orgProducts
     * @return array
     */
    public function getPrimaryOrganizationInfo($nrOrg, $orgProducts) {
        $organization = RepositoryInstance::getOrganizacao()->getOrganizacaoByNrorg($nrOrg);
        $primaryOrgId = $nrOrg;

        if(!empty($organization) && !empty($organization['NRORGPRINCIPAL'])) {
            $primaryOrgId = $organization['NRORGPRINCIPAL'];
            $primaryOrgProducts = RepositoryInstance::getOmProdutoElemento()->getProductsWithElementsByNrOrgId($primaryOrgId, true);
            $orgProductIds = array_map(function($orgProduct) {
                return $orgProduct['productId'];
            }, $orgProducts);
            foreach($primaryOrgProducts as $primaryOrgProduct) {
                if(!in_array($primaryOrgProduct['productId'], $orgProductIds)) {
                    $orgProducts[] = $primaryOrgProduct;
                }
            }
        }

        $relatedOrgs   = RepositoryInstance::getOrganizacao()->getOrganizacaoByNrorgPrincipal($primaryOrgId);
        $relatedOrgIds = Utilities::getArrayFromAssociativeArray($relatedOrgs, 'NRORG');

        return array(
            'ORG_PRODUCTS'    => $orgProducts,
            'RELATED_ORG_IDS' => $relatedOrgIds
        );
    }

    public function saveOrgFilial($organizationId, $cdFilial)
    {
        $entityManager = GeneralInstance::getEntityManager();

        $findQuery = "
            SELECT ID 
              FROM CP_ORG_FILIAL 
             WHERE NRORG_ID = $organizationId 
               AND CDFILIAL = '$cdFilial'
        ";
        $registers = $entityManager->getConnection()->fetchAll($findQuery);

        if(empty($registers)) {
            $id = GeneralInstance::getNewCode()->getCodeByTableName('CP_ORG_FILIAL', 20, 1, 10, true);
            $operatorCode = '000000000997';
            $query = "
                INSERT INTO CP_ORG_FILIAL (ID, NRORG_ID, CDFILIAL, CDOPERINCLUSAO, CDOPERULTATU)
                                   VALUES ($id, $organizationId, $cdFilial, '$operatorCode', '$operatorCode')
            ";
        } else {
            $query = "
                UPDATE CP_ORG_FILIAL
                   SET DTULTATU = SYSDATE
                 WHERE NRORG_ID = $organizationId 
                   AND CDFILIAL = '$cdFilial'
            ";
        }

        $entityManager->getConnection()->executeQuery($query);
    }

    public function getOrgFileServerKey($organizationId)
    {
        $organization = RepositoryInstance::getOrganizacao()->getOrganizacaoByNrorg($organizationId);
        $isSaaSOrg    = $organization && $organization['IDORGSAAS'] != 'N';

        if($isSaaSOrg) {
            return ServiceInstance::getOrganizationHandler()->getAPIKey($organizationId);
        } else {
            throw new \Exception('A chave do File Server só pode ser gerada pelo License para Organizações SaaS.');
        }
    }

    public function getOrgProducts($organizationId)
    {
        $omProdutoRepository = RepositoryInstance::getOmProduto();
        $products = $omProdutoRepository->getOrgProducts($organizationId);
        foreach($products as &$product) {
            $product['VERSIONS'] = $omProdutoRepository->getProductAvailableVersions($product['ID'], $organizationId);
        }
        return $products;
    }
}
